import sys, os
import math
from PyQt6.QtWidgets import QApplication, QWidget, QVBoxLayout, QLabel, QFrame, QSizePolicy, QHBoxLayout, QToolTip
from PyQt6.QtCore import QTimer, QDateTime, Qt, QTimeZone, QPoint, QTime, QRect
from PyQt6.QtGui import QFont, QFontMetrics, QPainter, QPen, QPixmap, QPainterPath, QBrush, QColor, QCursor
from jhora.panchanga import drik
from jhora import utils, const

# Define constants for various parameters of Analog Clock
# 1 day = 60 ghati; 1 Ghati = 24 minutes = 60 Phala; 1 Phala = 24 seconds = 60 Vighati; 1 Vighat = 0.4 second
# 1 day = 30 muhurtha; 1 MUHURTHA = 48 minutes = 30 Kaala; 1 Kaala = 1.6 minutes = 96 seconds = 30 Kaastha; 1 Kaastha = 3.2 seconds 
_VEDIC_HOURS_PER_DAY = 60#60 #60=> Ghatis 60 Ghati = 1 day; 30=>30 Muhurthas = 1 day
_HALF_VEDIC_HOURS_PER_DAY = _VEDIC_HOURS_PER_DAY/2
_CLOCK_SIZE = 500
_CENTER_X = 250
_CENTER_Y = 250
_DAY_COLOR = QColor(255, 255, 224, 127)
_NIGHT_COLOR = QColor(211, 211, 211, 127)
_DAY_NIGHT_ANGLE_OFFSET = 80
_OUTER_CIRCLE_RADIUS = 180
_CAPTION_RADIUS_INCREMENT = 15
_CAPTION_ANGLE_SHIFT = 15 # degrees
_CAPTION_COLOR = Qt.GlobalColor.darkBlue
_PLACE_LABEL_COLOR = Qt.GlobalColor.darkRed
_OUTER_LABEL_STEP =  5 if _VEDIC_HOURS_PER_DAY==60 else 1
_OUTER_LABEL_COLOR = Qt.GlobalColor.darkMagenta
_OUTER_TICK_STEP = 1
_INNER_CIRCLE_RADIUS = 100
_INNER_LABEL_STEP = 5 if _VEDIC_HOURS_PER_DAY==60 else 1
_INNER_LABEL_COLOR = Qt.GlobalColor.darkGreen
_INNER_TICK_STEP = 1
_TICK_MARK_LENGTH = 10
_LABEL_TICK_MARK_LENGTH = 20
_LABEL_OFFSET = 20
# _HALF_VEDIC_HOURS_PER_DAY (15 or 30) if East is in the left. 0 if sunrise starts from right (East in the right) or  
_LABEL_AT_ZERO_DEGREES = int(_HALF_VEDIC_HOURS_PER_DAY)
# Below is the actual angle in degrees at which Sunrise is. 180 if East in the left or 0 if East in the right
_SUNRISE_LABEL_DEGREES = 180
_HAND_PEN_WIDTH = 3
_GHATI_HAND_LENGTH = 150
_GHATI_HAND_COLOR = Qt.GlobalColor.red
_PHALA_HAND_LENGTH = 170
_PHALA_HAND_COLOR = Qt.GlobalColor.blue
_VIGHATI_HAND_LENGTH = 80
_VIGHATI_HAND_COLOR = Qt.GlobalColor.green
_TIMER_FREQUENCY = 400  # = one second
_ANGLE_FACTOR = 360/_VEDIC_HOURS_PER_DAY  # (360/60)
_IMAGE_ICON = os.path.abspath(const._IMAGES_PATH) + const._sep + "lord_ganesha1.jpg"
_SUNRISE_ICON = os.path.abspath(const._IMAGES_PATH) + const._sep + "sunrise.png"
_SUNSET_ICON = os.path.abspath(const._IMAGES_PATH) + const._sep + "sunset.png"
_ICON_SIZE = 64
_DAYNIGHT_STR_COLOR = Qt.GlobalColor.darkGreen
_day_night_radius_factor = 0.7

class VedicAnalogClock(QWidget):
    """
        force_equal_day_night_ghati = True will force equal 30 ghatis for day and night.
        This feature is for experimental purpose. 
        Some panchang websites like drikpanchang may force 30 ghatis for both day and night
        so that sunset is always equals to 30 ghati. 
        But traditionally vedic praharas are unequal when day/nights are unequal
        So use this function with caution. 
        Also enabling this feature in vedic clock will show unqual hand movements
    """

    def __init__(self, jd=None, place=None,language='ta',force_equal_day_night_ghati=False):
        super().__init__()
        utils.set_language(language)
        self.res = utils.resource_strings
        self.setMouseTracking(True)
        self.jd = jd; self.place = place
        self.force_equal_day_night_ghati = force_equal_day_night_ghati
        self.captions = [self.res[cap+'_str'] for cap in const.periods_of_the_day]
        self._day_night_str = [self.res['daytime_str'],self.res['nighttime_str']]
        #self.current_datetime = QDateTime.currentDateTime()
        self._get_drik_info()
        self.setWindowTitle(self.res['vedic_clock_str'])
        self.setFixedSize(_CLOCK_SIZE, _CLOCK_SIZE)
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.update)
        self.timer.start(_TIMER_FREQUENCY)  # Update every second
        # Load the icon
        self.icon = QPixmap(_IMAGE_ICON).scaled(_ICON_SIZE, _ICON_SIZE, Qt.AspectRatioMode.KeepAspectRatio)
        self.painter = QPainter(self)
        self.show()
        self.repaint()
    def mouseMoveEvent(self, event):
        pos = event.pos()
        if abs(pos.x()-self.sunrise_icon_pos[0])<=self.sunrise_icon.width() and abs(pos.y()-self.sunrise_icon_pos[1])<=self.sunrise_icon.height():
            srise_tip = self.res['sunrise_str'] + ' ('+utils.to_dms(self._today_sunrise)+')'
            QToolTip.showText(QCursor.pos(), srise_tip, self)
        elif abs(pos.x()-self.sunset_icon_pos[0])<=self.sunset_icon.width() and abs(pos.y()-self.sunset_icon_pos[1])<=self.sunset_icon.height():
            sset_tip = self.res['sunset_str'] + ' ('+utils.to_dms(self._today_sunset)+')'
            QToolTip.showText(QCursor.pos(), sset_tip, self)
        else:
            QToolTip.hideText()
    def closeEvent(self, *args, **kwargs):
        self.timer.stop()
        return QWidget.closeEvent(self, *args, **kwargs)
    def _get_current_date_time(self):
        current_datetime = QDateTime.currentDateTime()
        #self.current_datetime = self.current_datetime.addSecs(30*60)
        local_time_zone_hours = self.place.timezone
        time_zone = QTimeZone(int(local_time_zone_hours * 3600))
        #return self.current_datetime.toTimeZone(time_zone)
        return current_datetime.toTimeZone(time_zone)
    def _get_drik_info(self):
        if self.place is None:
            loc = utils.get_place_from_user_ip_address()
            self.place = drik.Place(loc[0], loc[1], loc[2], loc[3])
        current_datetime = self._get_current_date_time()
        if self.jd is None:
            self.jd = utils.julian_day_number(drik.Date(current_datetime.date().year(),
                                                        current_datetime.date().month(),
                                                        current_datetime.date().day()),
                                              (current_datetime.time().hour(),
                                               current_datetime.time().minute(),
                                               current_datetime.time().second()))
        self._day_length = drik.day_length(self.jd, self.place); self._night_length = drik.night_length(self.jd, self.place)
        _day_duration = self._day_length + self._night_length
        self._vedic_hour_per_local_hour = _VEDIC_HOURS_PER_DAY / _day_duration
        self._today_sunrise = drik.sunrise(self.jd, self.place)[0]; self._today_sunset = drik.sunset(self.jd,self.place)[0]
        self._tomorrow_sunrise = drik.sunrise(self.jd+1, self.place)[0]
        # Convert self._tomorrow_sunrise (float hours) to QDateTime
        tomorrow_date = current_datetime.addDays(1).date()
        tomorrow_sunrise_time = QTime(
            int(self._tomorrow_sunrise),
            int((self._tomorrow_sunrise - int(self._tomorrow_sunrise)) * 60)
        )
        local_time_zone_hours = self.place.timezone
        time_zone = QTimeZone(int(local_time_zone_hours * 3600))
        self.qtime_tomorrow_sunrise = QDateTime(tomorrow_date, tomorrow_sunrise_time,time_zone)
        self._day_vedic_hour_per_local_hour = _HALF_VEDIC_HOURS_PER_DAY / self._day_length
        self._night_vedic_hour_per_local_hour = _HALF_VEDIC_HOURS_PER_DAY / self._night_length
        ssvt = self.get_vedic_time_equal_day_night_ghati(self._today_sunset) if self.force_equal_day_night_ghati \
                                            else self.get_vedic_time(self._today_sunset)
        ssgt = ssvt[0]+ssvt[1]/_VEDIC_HOURS_PER_DAY+ssvt[2]/_VEDIC_HOURS_PER_DAY/_VEDIC_HOURS_PER_DAY
        ssdeg = ssgt*_ANGLE_FACTOR
        self._sunset_degrees = math.ceil(ssdeg)
    def get_vedic_time_equal_day_night_ghati(self,float_hours=None):
        # Get the current date and time
        current_datetime = self._get_current_date_time()
        # Add the check for tomorrow's sunrise
        if current_datetime >= self.qtime_tomorrow_sunrise:
            self.jd += 1
            self._get_drik_info()
        if float_hours is None: 
            current_datetime = self._get_current_date_time()
            float_hours = current_datetime.time().hour() + current_datetime.time().minute() / 60 + \
                      current_datetime.time().second() / 3600 +current_datetime.time().msec() / 3600000
        if float_hours <= self._today_sunset and float_hours >= self._today_sunrise:
            _vedic_hours = float_hours - self._today_sunrise
            if _vedic_hours < 0:
                _vedic_hours += 24
            total_vedic_hours = _vedic_hours * self._day_vedic_hour_per_local_hour
        else:
            total_vedic_hours = _HALF_VEDIC_HOURS_PER_DAY + (float_hours-self._today_sunset)*self._night_vedic_hour_per_local_hour if float_hours>=self._today_sunset \
                            else _VEDIC_HOURS_PER_DAY - (self._today_sunrise-float_hours)*self._night_vedic_hour_per_local_hour
        vedic_hours = int(total_vedic_hours)
        vedic_minutes = int((total_vedic_hours - vedic_hours) * _VEDIC_HOURS_PER_DAY)
        vedic_seconds = int(((total_vedic_hours - vedic_hours) * _VEDIC_HOURS_PER_DAY - vedic_minutes) * _VEDIC_HOURS_PER_DAY)
    
        return int(vedic_hours), int(vedic_minutes), int(vedic_seconds)
    def get_vedic_time(self, float_hours=None):
        # Get the current date and time
        current_datetime = self._get_current_date_time()
        # Add the check for tomorrow's sunrise
        if current_datetime >= self.qtime_tomorrow_sunrise:
            self.jd += 1
            self._get_drik_info()
        
        if float_hours is None:
            float_hours = current_datetime.time().hour() + current_datetime.time().minute() / 60 + \
                          current_datetime.time().second() / 3600 + current_datetime.time().msec() / 3600000
    
        local_hours_since_sunrise = float_hours - self._today_sunrise
        if local_hours_since_sunrise < 0:
            local_hours_since_sunrise += 24
        
        total_vedic_hours = local_hours_since_sunrise * self._vedic_hour_per_local_hour
            
        vedic_hours = int(total_vedic_hours)
        vedic_minutes = int((total_vedic_hours - vedic_hours) * _VEDIC_HOURS_PER_DAY)
        vedic_seconds = int(((total_vedic_hours - vedic_hours) * _VEDIC_HOURS_PER_DAY - vedic_minutes) * _VEDIC_HOURS_PER_DAY)
        return vedic_hours, vedic_minutes, vedic_seconds
    def paintEvent(self, event):
        try:
            painter = QPainter(self)
            painter.setRenderHint(QPainter.RenderHint.Antialiasing)
    
            # Calculate available height for the clock drawing
            available_height = self.height() - 50  # Reserve 50 pixels for text at the bottom
    
            # Draw the background semicircles
            self.drawCircles(painter)
            # Draw fixed captions
            self.drawFixedCaptions(painter)
    
            # Paint the big circle for ghati and phala
            self.drawLabels(painter, _CENTER_X, _CENTER_Y, _OUTER_CIRCLE_RADIUS, _OUTER_LABEL_STEP, _OUTER_LABEL_COLOR)  # Outer labels
            self.drawTickMarks(painter, _CENTER_X, _CENTER_Y, _OUTER_CIRCLE_RADIUS, _OUTER_TICK_STEP)  # Outer tick marks
    
            # Draw the tick marks
            self.drawTickMarks(painter, _CENTER_X, _CENTER_Y, _OUTER_CIRCLE_RADIUS, _OUTER_TICK_STEP)
    
            # Calculate angles for clock hands
            vedic_hours, vedic_minutes, vedic_seconds = self.get_vedic_time_equal_day_night_ghati() if self.force_equal_day_night_ghati \
                                            else self.get_vedic_time()
    
            # Draw ghati hand (small) - Red
            vedic_hour_angle = _ANGLE_FACTOR * vedic_hours
            self.drawHand(painter, _CENTER_X, _CENTER_Y, _GHATI_HAND_LENGTH, vedic_hour_angle, _GHATI_HAND_COLOR)
    
            # Draw phala hand (big) - Blue
            vedic_minute_angle = _ANGLE_FACTOR * vedic_minutes
            self.drawHand(painter, _CENTER_X, _CENTER_Y, _PHALA_HAND_LENGTH, vedic_minute_angle, _PHALA_HAND_COLOR)
    
            self.drawLabels(painter, _CENTER_X, _CENTER_Y, _INNER_CIRCLE_RADIUS, _INNER_LABEL_STEP, _INNER_LABEL_COLOR)  # Inner labels
            self.drawTickMarks(painter, _CENTER_X, _CENTER_Y, _INNER_CIRCLE_RADIUS, _INNER_TICK_STEP)  # Inner tick marks
    
            # Draw vighati hand
            vedic_second_angle = _ANGLE_FACTOR * vedic_seconds
            self.drawHand(painter, _CENTER_X, _CENTER_Y, _VIGHATI_HAND_LENGTH, vedic_second_angle, _VIGHATI_HAND_COLOR)
            # Draw the icon at the center
            icon_size = self.icon.size()
            icon_x = _CENTER_X - icon_size.width() // 2
            icon_y = _CENTER_Y - icon_size.height() // 2
            # Create a circular mask
            mask = QPainterPath()
            mask.addEllipse(icon_x, icon_y, icon_size.width(), icon_size.height())
            painter.setClipPath(mask)
            
            # Draw the masked icon
            painter.drawPixmap(icon_x, icon_y, self.icon.scaled(_ICON_SIZE, _ICON_SIZE, Qt.AspectRatioMode.KeepAspectRatioByExpanding))
            
            if self.place is not None:
                key1 = self.res['place_str']+': '
                value1 = self.place.Place
                _lat = utils.to_dms(float(self.place.latitude), is_lat_long='lat')
                _long = utils.to_dms(float(self.place.longitude), is_lat_long='long')
                value1 += ' (' + _lat + ', ' + _long + ')'
                year, month, day,_ = utils.jd_to_gregorian(self.jd)
                date_str1 = str(year)+','+str(month)+','+str(day)
                value2 = self.res['date_of_birth_str']+' : '+ date_str1
                text = key1 + value1+'\n'+value2
                painter.setPen(QPen(_PLACE_LABEL_COLOR))
                # Ensure the text is within the visible area
                text_rect = QRect(0, self.height() - 50, self.width(), 50)
                painter.setClipRect(text_rect)  # Clip to ensure the text is visible
                painter.drawText(text_rect, Qt.AlignmentFlag.AlignCenter, text)
                painter.setPen(QPen())
        except Exception as e:
            print(f"Error during paint event: {e}")
    def drawCircles(self,painter):
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)

        # Draw the outer circle
        painter.setBrush(_NIGHT_COLOR)  # very light grey
        painter.drawEllipse(_CENTER_X - _OUTER_CIRCLE_RADIUS, _CENTER_Y - _OUTER_CIRCLE_RADIUS, 
                            2 * _OUTER_CIRCLE_RADIUS, 2 * _OUTER_CIRCLE_RADIUS)

        # Draw the annular region
        painter.setBrush(_DAY_COLOR)  # light yellow
        start_angle = _SUNRISE_LABEL_DEGREES * 16
        sweep_length = - self._sunset_degrees * 16
        painter.drawPie(_CENTER_X - _OUTER_CIRCLE_RADIUS, _CENTER_Y - _OUTER_CIRCLE_RADIUS,
                        2 * _OUTER_CIRCLE_RADIUS, 2 * _OUTER_CIRCLE_RADIUS, start_angle, sweep_length)

        # Draw the inner circle
        painter.setBrush(self.palette().window().color())  # fill with the widget's background color
        painter.drawEllipse(_CENTER_X - _INNER_CIRCLE_RADIUS, _CENTER_Y - _INNER_CIRCLE_RADIUS, 
                            2 * _INNER_CIRCLE_RADIUS, 2 * _INNER_CIRCLE_RADIUS)
        
        # Draw "Day" label
        day_angle = (_SUNRISE_LABEL_DEGREES+_DAY_NIGHT_ANGLE_OFFSET); day_angle_rad = day_angle * (math.pi / 180)
        day_x = int(_CENTER_X + (_INNER_CIRCLE_RADIUS + _day_night_radius_factor*(_OUTER_CIRCLE_RADIUS - _INNER_CIRCLE_RADIUS) / 2) * math.cos(day_angle_rad))
        day_y = int(_CENTER_Y + (_INNER_CIRCLE_RADIUS + _day_night_radius_factor*(_OUTER_CIRCLE_RADIUS - _INNER_CIRCLE_RADIUS) / 2) * math.sin(day_angle_rad))
        painter.setPen(QPen(_DAYNIGHT_STR_COLOR))
        painter.drawText(day_x, day_y, self._day_night_str[0])
        # Draw "Night" label
        night_angle_rad = (180 + day_angle+(90 -_DAY_NIGHT_ANGLE_OFFSET)) * (math.pi / 180)
        night_x = int(_CENTER_X + (_INNER_CIRCLE_RADIUS + _day_night_radius_factor*(_OUTER_CIRCLE_RADIUS - _INNER_CIRCLE_RADIUS) / 2) * math.cos(night_angle_rad))
        night_y = int(_CENTER_Y + (_INNER_CIRCLE_RADIUS + _day_night_radius_factor*(_OUTER_CIRCLE_RADIUS - _INNER_CIRCLE_RADIUS) / 2) * math.sin(night_angle_rad))
        painter.setPen(QPen(_DAYNIGHT_STR_COLOR))
        painter.drawText(night_x, night_y, self._day_night_str[1])

    # Draw _SUNRISE_ICON
        self.sunrise_icon = QPixmap(_SUNRISE_ICON)
        sunrise_x = int(_CENTER_X + (_OUTER_CIRCLE_RADIUS + 20) * math.cos(start_angle / 16 * math.pi / 180))
        sunrise_y = int(_CENTER_Y + (_OUTER_CIRCLE_RADIUS + 20) * math.sin(start_angle / 16 * math.pi / 180))
        self.sunrise_icon_pos = (int(sunrise_x) - self.sunrise_icon.width() // 2,
                                    int(sunrise_y) - self.sunrise_icon.height() // 2)
        painter.drawPixmap(QPoint(self.sunrise_icon_pos[0],self.sunrise_icon_pos[1]), self.sunrise_icon)
        # Correct the calculation of the sunset angle
        sunset_angle = (_SUNRISE_LABEL_DEGREES + self._sunset_degrees) * 16
        self.sunset_icon = QPixmap(_SUNSET_ICON)
        sunset_x = int(_CENTER_X + (_OUTER_CIRCLE_RADIUS + 20) * math.cos(sunset_angle / 16 * math.pi / 180))
        sunset_y = int(_CENTER_Y + (_OUTER_CIRCLE_RADIUS + 20) * math.sin(sunset_angle / 16 * math.pi / 180))
        self.sunset_icon_pos = (int(sunset_x) - self.sunset_icon.width() // 2,
                                    int(sunset_y) - self.sunset_icon.height() // 2)
        painter.drawPixmap(QPoint(self.sunset_icon_pos[0],self.sunset_icon_pos[1]), self.sunset_icon)
    def drawTickMarks(self, painter, centerX, centerY, radius, step):
        for i in range(0, _VEDIC_HOURS_PER_DAY, step):
            angle = _ANGLE_FACTOR * i
            x1 = centerX + int(radius * math.cos(math.radians(angle)))
            y1 = centerY + int(radius * math.sin(math.radians(angle)))
            if i % 5 == 0:  # Label tick mark
                x2 = centerX + int((radius - _LABEL_TICK_MARK_LENGTH) * math.cos(math.radians(angle)))
                y2 = centerY + int((radius - _LABEL_TICK_MARK_LENGTH) * math.sin(math.radians(angle)))
            else:  # Regular tick mark
                x2 = centerX + int((radius - _TICK_MARK_LENGTH) * math.cos(math.radians(angle)))
                y2 = centerY + int((radius - _TICK_MARK_LENGTH) * math.sin(math.radians(angle)))
            painter.drawLine(x1, y1, x2, y2)

    def drawLabels(self, painter, centerX, centerY, radius, step,label_color):
        painter.setPen(QPen(label_color))
        font = QFont(); font.setBold(True); painter.setFont(font)
        for i in range(0, _VEDIC_HOURS_PER_DAY+1, step):
            angle = _ANGLE_FACTOR * i
            x = centerX + int((radius - _LABEL_OFFSET) * math.cos(math.radians(angle))) - 10
            y = centerY + int((radius - _LABEL_OFFSET) * math.sin(math.radians(angle))) + 10
            label = (i + _LABEL_AT_ZERO_DEGREES) % _VEDIC_HOURS_PER_DAY
            painter.drawText(x, y, f"{label}")
        
    def drawFixedCaptions(self, painter):
        num_captions = len(self.captions)
        angle_step = 360 / num_captions
        painter.setPen(QPen(_CAPTION_COLOR))
        for i in range(num_captions):
            angle = math.radians(_SUNRISE_LABEL_DEGREES + i * angle_step+_CAPTION_ANGLE_SHIFT)
            angle_degrees = i * angle_step

            # Calculate the position just outside the outer circle
            x = _CENTER_X + int((_OUTER_CIRCLE_RADIUS + _CAPTION_RADIUS_INCREMENT) * math.cos(angle))
            y = _CENTER_Y + int((_OUTER_CIRCLE_RADIUS + _CAPTION_RADIUS_INCREMENT) * math.sin(angle))

            # Save the painter state
            painter.save()

            # Translate and rotate the painter to align text with the circle's perimeter
            painter.translate(x, y)
            
            if angle_degrees <= 180:
                painter.rotate(angle_degrees - 90)
            else:
                painter.rotate(angle_degrees +  90)
            # Draw the text
            painter.drawText(-10, 0, self.captions[i])  # Adjust to center text

            # Restore the painter state
            painter.restore()
    def drawHand(self, painter, centerX, centerY, length, angle, color, arrowSize=10):
        pen = QPen(color, _HAND_PEN_WIDTH)
        painter.setPen(pen)
        
        # Calculate the end point of the arrow
        endX = centerX + int(length * math.cos(math.radians(angle + _SUNRISE_LABEL_DEGREES)))
        endY = centerY + int(length * math.sin(math.radians(angle + _SUNRISE_LABEL_DEGREES)))
    
        # Draw the shaft of the arrow
        painter.drawLine(centerX, centerY, endX, endY)
    
        # Calculate the two points for the arrowhead, making sure the tip points outward
        angleRad = math.radians(angle + _SUNRISE_LABEL_DEGREES)
        leftX = endX + int(arrowSize * math.cos(angleRad + math.pi - math.pi / 6))
        leftY = endY + int(arrowSize * math.sin(angleRad + math.pi - math.pi / 6))
        rightX = endX + int(arrowSize * math.cos(angleRad + math.pi + math.pi / 6))
        rightY = endY + int(arrowSize * math.sin(angleRad + math.pi + math.pi / 6))
    
        # Draw the arrowhead
        painter.drawLine(endX, endY, leftX, leftY)
        painter.drawLine(endX, endY, rightX, rightY)
        painter.drawLine(leftX, leftY, rightX, rightY)

################ Digital Clock ##############
_CLOCK_LABEL_WIDTH = 150
_CAPTION_FONT_SIZE = 12; _CLOCK_FONT_SIZE = 12
_LOCAL_CLOCK_COLOR = 'red'; _VEDIC_CLOCK_COLOR = 'green'

class VedicDigitalClock(QWidget):
    """
        force_equal_day_night_ghati = True will force equal 30 ghatis for day and night.
        This feature is for experimental purpose. 
        Some panchang websites like drikpanchang may force 30 ghatis for both day and night
        so that sunset is always equals to 30 ghati. 
        But traditionally vedic praharas are unequal when day/nights are unequal
        So use this function with caution. 
        Also enabling this feature in vedic clock will show unqual hand movements
    """
    def __init__(self, language='en', jd=None, place=None,adjust_label_widths=False,show_local_clock=False,
                 show_clock_caption=False,show_clock_label=True,horizontal_clocks=False,
                 force_equal_day_night_ghati=False):
        super().__init__()
        utils.set_language(language)
        self.res = utils.resource_strings
        self.force_equal_day_night_ghati = force_equal_day_night_ghati
        self.jd = jd; self.place = place
        self._get_drik_info()
        self.adjust_label_widths = adjust_label_widths
        self.show_local_clock = show_local_clock
        self.show_clock_caption = show_clock_caption; self.show_clock_label = show_clock_label
        self.horizontal_clocks = horizontal_clocks
        self.initUI()
    def initUI(self):
        layout = QHBoxLayout() if self.horizontal_clocks else QVBoxLayout()

        if self.show_clock_caption: 
            if self.show_local_clock:self.local_caption = QLabel(self.res['local_clock_str'], self)
            self.vedic_caption = QLabel(utils.resource_strings['vedic_clock_str'], self)
        if self.show_clock_label:
            if self.show_local_clock:self.local_format_label = QLabel(f"{self.res['hours_str']}:{self.res['minutes_str']}:{self.res['seconds_str']}", self)
            self.vedic_format_label = QLabel(f"{self.res['ghati_str']}:{self.res['pala_str']}:{self.res['vighati_str']}", self)
        if self.show_local_clock: self.local_time_label = QLabel(self)
        self.vedic_time_label = QLabel(self)
        if self.show_clock_caption and self.show_clock_label:
            for label in [self.local_caption, self.local_format_label, self.vedic_caption, self.vedic_format_label]:
                if label:
                    label.setFont(QFont("Arial", _CAPTION_FONT_SIZE, QFont.Weight.Bold))
                    label.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
                    label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.vedic_time_label.setFrameStyle(QFrame.Shape.Box | QFrame.Shadow.Raised)
        self.vedic_time_label.setLineWidth(2)
        self.vedic_time_label.setFont(QFont("Arial", _CLOCK_FONT_SIZE, QFont.Weight.Bold))
        self.vedic_time_label.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        self.vedic_time_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.vedic_time_label.setStyleSheet("color: "+_LOCAL_CLOCK_COLOR+"; min-width: 120px; max-width: 120px;")
        if self.show_local_clock:
            self.local_time_label.setFrameStyle(QFrame.Shape.Box | QFrame.Shadow.Raised)
            self.local_time_label.setLineWidth(2)
            self.local_time_label.setFont(QFont("Arial", _CLOCK_FONT_SIZE, QFont.Weight.Bold))
            self.local_time_label.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
            self.local_time_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            self.local_time_label.setStyleSheet("color: "+_LOCAL_CLOCK_COLOR+"; min-width: 120px; max-width: 120px;")
            separator = QFrame()
            separator.setFrameShape(QFrame.Shape.HLine)
            #separator.setFrameShadow(QFrame.Shadow.Sunken)
            #separator.sizePolicy().setHorizontalPolicy(QSizePolicy.Policy.Fixed)
        self.vedic_time_label.setStyleSheet("color: "+_VEDIC_CLOCK_COLOR+"; min-width: 120px; max-width: 120px;")
        if self.show_clock_caption: layout.addWidget(self.local_caption)
        if self.show_local_clock:
            if self.show_clock_label: layout.addWidget(self.local_format_label)
            layout.addWidget(self.local_time_label)
            layout.addWidget(separator)
        if self.show_clock_caption: layout.addWidget(self.vedic_caption)
        if self.show_clock_label: layout.addWidget(self.vedic_format_label)
        layout.addWidget(self.vedic_time_label)
        
        self.setLayout(layout)

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.update_time)
        self.timer.start(1000)
        self.setWindowTitle(self.res['vedic_clock_str'])
        self.show()
    def closeEvent(self, *args, **kwargs):
        self.timer.stop()
        return QWidget.closeEvent(self, *args, **kwargs)
    def _get_current_date_time(self):
        current_datetime = QDateTime.currentDateTime()
        local_time_zone_hours = self.place.timezone
        time_zone = QTimeZone(int(local_time_zone_hours * 3600))
        return current_datetime.toTimeZone(time_zone)
    def _get_drik_info(self):
        if self.place is None:
            loc = utils.get_place_from_user_ip_address()
            self.place = drik.Place(loc[0], loc[1], loc[2], loc[3])
        current_datetime = self._get_current_date_time()
        if self.jd is None:
            self.jd = utils.julian_day_number(drik.Date(current_datetime.date().year(),
                                                        current_datetime.date().month(),
                                                        current_datetime.date().day()),
                                              (current_datetime.time().hour(),
                                               current_datetime.time().minute(),
                                               current_datetime.time().second()))
        self._day_length = drik.day_length(self.jd, self.place); self._night_length = drik.night_length(self.jd, self.place)
        _day_duration = self._day_length + self._night_length
        self._vedic_hour_per_local_hour = _VEDIC_HOURS_PER_DAY / _day_duration
        self._today_sunrise = drik.sunrise(self.jd, self.place)[0]; self._today_sunset = drik.sunset(self.jd,self.place)[0]
        self._tomorrow_sunrise = drik.sunrise(self.jd+1, self.place)[0]
        # Convert self._tomorrow_sunrise (float hours) to QDateTime
        tomorrow_date = current_datetime.addDays(1).date()
        tomorrow_sunrise_time = QTime(
            int(self._tomorrow_sunrise),
            int((self._tomorrow_sunrise - int(self._tomorrow_sunrise)) * 60)
        )
        local_time_zone_hours = self.place.timezone
        time_zone = QTimeZone(int(local_time_zone_hours * 3600))
        self.qtime_tomorrow_sunrise = QDateTime(tomorrow_date, tomorrow_sunrise_time,time_zone)
        self._day_vedic_hour_per_local_hour = _HALF_VEDIC_HOURS_PER_DAY / self._day_length
        self._night_vedic_hour_per_local_hour = _HALF_VEDIC_HOURS_PER_DAY / self._night_length
        ssvt = self.get_vedic_time_equal_day_night_ghati(self._today_sunset) if self.force_equal_day_night_ghati \
                                            else self.get_vedic_time(self._today_sunset)
        ssgt = ssvt[0]+ssvt[1]/_VEDIC_HOURS_PER_DAY+ssvt[2]/_VEDIC_HOURS_PER_DAY/_VEDIC_HOURS_PER_DAY
        ssdeg = ssgt*_ANGLE_FACTOR
        self._sunset_degrees = math.ceil(ssdeg)
    def get_vedic_time_equal_day_night_ghati(self,float_hours=None):
        # Get the current date and time
        current_datetime = self._get_current_date_time()
        # Add the check for tomorrow's sunrise
        if current_datetime >= self.qtime_tomorrow_sunrise:
            self.jd += 1
            self._get_drik_info()
        if float_hours is None: 
            current_datetime = self._get_current_date_time()
            float_hours = current_datetime.time().hour() + current_datetime.time().minute() / 60 + \
                      current_datetime.time().second() / 3600 +current_datetime.time().msec() / 3600000
        if float_hours <= self._today_sunset and float_hours >= self._today_sunrise:
            _vedic_hours = float_hours - self._today_sunrise
            if _vedic_hours < 0:
                _vedic_hours += 24
            total_vedic_hours = _vedic_hours * self._day_vedic_hour_per_local_hour
        else:
            total_vedic_hours = _HALF_VEDIC_HOURS_PER_DAY + (float_hours-self._today_sunset)*self._night_vedic_hour_per_local_hour if float_hours>=self._today_sunset \
                            else _VEDIC_HOURS_PER_DAY - (self._today_sunrise-float_hours)*self._night_vedic_hour_per_local_hour
        vedic_hours = int(total_vedic_hours)
        vedic_minutes = int((total_vedic_hours - vedic_hours) * _VEDIC_HOURS_PER_DAY)
        vedic_seconds = int(((total_vedic_hours - vedic_hours) * _VEDIC_HOURS_PER_DAY - vedic_minutes) * _VEDIC_HOURS_PER_DAY)
    
        return int(vedic_hours), int(vedic_minutes), int(vedic_seconds)
    def get_vedic_time(self, float_hours=None):
        # Get the current date and time
        current_datetime = self._get_current_date_time()
        # Add the check for tomorrow's sunrise
        if current_datetime >= self.qtime_tomorrow_sunrise:
            self.jd += 1
            self._get_drik_info()
        
        if float_hours is None:
            float_hours = current_datetime.time().hour() + current_datetime.time().minute() / 60 + \
                          current_datetime.time().second() / 3600 + current_datetime.time().msec() / 3600000
    
        local_hours_since_sunrise = float_hours - self._today_sunrise
        if local_hours_since_sunrise < 0:
            local_hours_since_sunrise += 24
        
        total_vedic_hours = local_hours_since_sunrise * self._vedic_hour_per_local_hour
            
        vedic_hours = int(total_vedic_hours)
        vedic_minutes = int((total_vedic_hours - vedic_hours) * _VEDIC_HOURS_PER_DAY)
        vedic_seconds = int(((total_vedic_hours - vedic_hours) * _VEDIC_HOURS_PER_DAY - vedic_minutes) * _VEDIC_HOURS_PER_DAY)
        return vedic_hours, vedic_minutes, vedic_seconds
    def update_time(self):
        current_datetime = self._get_current_date_time()
        # Calculate angles for clock hands
        vedic_time = self.get_vedic_time_equal_day_night_ghati() if self.force_equal_day_night_ghati \
                                        else self.get_vedic_time()
        self.vedic_time_label.setText(f'{vedic_time[0]:02}:{vedic_time[1]:02}:{vedic_time[2]:02}')
        if self.show_local_clock: self.local_time_label.setText(current_datetime.time().toString("HH:mm:ss"))
    def adjust_font_size_to_width(self, label):
        font = label.font()
        font_metrics = QFontMetrics(font)
        max_width = _CLOCK_LABEL_WIDTH  # Ensure the font fits within the clock's width
        while font_metrics.horizontalAdvance(label.text()) > max_width:
            font.setPointSize(font.pointSize() - 1)
            font_metrics = QFontMetrics(font)
        label.setFont(font)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    dob = drik.Date(1996,12,7); tob = (23,59,59); place = drik.Place('Chennai,India',13.0878,80.2785,5.5)
    jd = utils.julian_day_number(dob,tob)
    jd = None; place = None
    #clock_widget = VedicDigitalClock(language='ta', jd=jd,place=place,show_local_clock=True)#,force_equal_day_night_ghati=True)
    clock_widget = VedicAnalogClock(language='ta', jd=jd,place=place)#,force_equal_day_night_ghati=True)
    sys.exit(app.exec())



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from PyQt6.QtWidgets import QLineEdit, QApplication, QLabel, QHBoxLayout, QVBoxLayout, QPushButton,\
                            QComboBox, QDialog, QListWidget, QAbstractScrollArea
from PyQt6.QtCore import Qt, QSize
from PyQt6.QtGui import QFontMetrics
from jhora import utils
from jhora.panchanga import drik, vratha
import datetime
_main_window_width = 550
_main_window_height = 550
_CALCULATE_WAIT_MSG = '<b>Calculating... Results will be updated here. Please wait...</b>'
class VrathaFinderDialog(QDialog):
    """
        Search Vratha Dialog
    """
    def __init__(self,current_date_jd,place:drik.Place,chart_title='Search for special days'):
        super().__init__()
        self._current_date_jd = current_date_jd
        self._selection_date_jd = current_date_jd
        self.Place = place
        self._chart_title = chart_title
        self._accept_clicked = False
        self.res = utils.resource_strings
        self._vratha_result_dates = []
        self._tithi_list = [self.res['tithi_str']]+ [utils.PAKSHA_LIST[0]+'-'+t for t in utils.TITHI_LIST[0:14]]+ \
                      [utils.TITHI_LIST[14]]+[utils.PAKSHA_LIST[1]+'-'+t for t in utils.TITHI_LIST[0:14]]+ \
                                   [utils.TITHI_LIST[29]]
        self._nakshathra_list = [self.res['nakshatra_str']]+utils.NAKSHATRA_LIST
        self._yogam_list = [self.res['yogam_str']]+utils.YOGAM_LIST
        self._tm_list = [self.res['tamil_month_str']]+utils.MONTH_LIST
        self._create_ui()
    def _create_ui(self):
        v_layout = QVBoxLayout()
        h_layout = QHBoxLayout()
        self._special_combo = QComboBox()
        self._special_combo.addItems([self.res[k+'_str'] for k in list(vratha.special_vratha_map.keys())+['customized']])
        self._special_combo.currentIndexChanged.connect(self._enable_combos)
        # Calculate the required width for the longest item
        font_metrics = QFontMetrics(self._special_combo.font())
        max_width = max(font_metrics.horizontalAdvance(self._special_combo.itemText(i)) for i in range(self._special_combo.count()))
        # Set the minimum width of the combo box to fit the longest item
        self._special_combo.setMinimumWidth(max_width + 50)  # Add some padding
        h_layout.addWidget(self._special_combo)
        self._from_date_label = QLabel(self.res['starts_at_str']); h_layout.addWidget(self._from_date_label)
        y,m,d,_ = utils.jd_to_gregorian(self._current_date_jd)
        self._from_date_text = QLineEdit(str(y)+','+str(m)+','+str(d));h_layout.addWidget(self._from_date_text)
        self._to_date_label = QLabel(self.res['ends_at_str']); h_layout.addWidget(self._to_date_label)
        y,m,d,_ = utils.jd_to_gregorian(self._current_date_jd+365)
        self._to_date_text = QLineEdit(str(y)+','+str(m)+','+str(d));h_layout.addWidget(self._to_date_text)
        v_layout.addLayout(h_layout)
        h_layout = QHBoxLayout()
        v_layout.addLayout(h_layout)
        h_layout = QHBoxLayout()
        self._tithi_combo = QComboBox()
        self._tithi_combo.addItems(self._tithi_list)
        h_layout.addWidget(self._tithi_combo)
        self._nakshatra_combo = QComboBox()
        self._nakshatra_combo.addItems(self._nakshathra_list)
        h_layout.addWidget(self._nakshatra_combo)
        self._yogam_combo = QComboBox()
        self._yogam_combo.addItems(self._yogam_list)
        h_layout.addWidget(self._yogam_combo)
        self._tm_combo = QComboBox()
        self._tm_combo.addItems(self._tm_list)
        h_layout.addWidget(self._tm_combo)
        v_layout.addLayout(h_layout)
        self._tithi_combo.setVisible(False); self._nakshatra_combo.setVisible(False)
        self._yogam_combo.setVisible(False); self._tm_combo.setVisible(False)
        self._tithi_combo.setEnabled(False); self._nakshatra_combo.setEnabled(False)
        self._yogam_combo.setEnabled(False); self._tm_combo.setEnabled(False)
        self._tithi_combo.currentIndexChanged.connect(self._enable_combos)
        self._nakshatra_combo.currentIndexChanged.connect(self._enable_combos)
        self._yogam_combo.currentIndexChanged.connect(self._enable_combos)
        self._tm_combo.currentIndexChanged.connect(self._enable_combos)
        self._results_list = QListWidget()
        #self._results_list.setEnabled(False)
        self._results_list.setSizeAdjustPolicy(QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)
        v_layout.addWidget(self._results_list)
        self._stats_label = QLabel('')
        v_layout.addWidget(self._stats_label)
        h_layout = QHBoxLayout()
        self._find_button = QPushButton(self.res['compute_str']); h_layout.addWidget(self._find_button)
        self._find_button.clicked.connect(self._calculate_clicked)
        self._accept_button = QPushButton(self.res['accept_str']); h_layout.addWidget(self._accept_button)
        self._accept_button.clicked.connect(self._accept_and_close)
        self._close_button = QPushButton(self.res['cancel_str']); h_layout.addWidget(self._close_button)
        self._close_button.clicked.connect(self._close_dialog)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
        self.setMinimumSize(_main_window_width,_main_window_height)
        self.setWindowTitle(self._chart_title)
        self._accept_button.setEnabled(False)
        self._calculate_clicked()
    def closeEvent(self, *args, **kwargs):
        self._selection_date_jd = self._current_date_jd
        QApplication.restoreOverrideCursor()
        return QDialog.closeEvent(self, *args, **kwargs)
    def _enable_combos(self):
        _width = self._special_combo.minimumSizeHint().width()
        self._special_combo.view().setMinimumWidth(_width+100)
        vrata_index = self._special_combo.currentIndex()
        self._results_list.clear();self._accept_button.setEnabled(False)
        if vrata_index == len(vratha.special_vratha_map.keys()):#vratha_type in ['tithi','nakshatra','yogam']:
            self._tithi_combo.setVisible(True); self._nakshatra_combo.setVisible(True)
            self._yogam_combo.setVisible(True); self._tm_combo.setVisible(True)
            self._tithi_combo.setEnabled(True); self._nakshatra_combo.setEnabled(True)
            self._yogam_combo.setEnabled(True); self._tm_combo.setEnabled(True)
        else:
            self._tithi_combo.setVisible(False); self._nakshatra_combo.setVisible(False)
            self._yogam_combo.setVisible(False); self._tm_combo.setVisible(False)
            self._tithi_combo.setEnabled(False); self._nakshatra_combo.setEnabled(False)
            self._yogam_combo.setEnabled(False); self._tm_combo.setEnabled(False)
        self._calculate_clicked()
    def _calculate_clicked(self):
        self._accept_button.setEnabled(False)
        _start_date = self._from_date_text.text().split(','); _start_date=drik.Date(int(_start_date[0]),int(_start_date[1]),int(_start_date[2]))
        _end_date = self._to_date_text.text().split(','); _end_date=drik.Date(int(_end_date[0]),int(_end_date[1]),int(_end_date[2]))
        vrata_index = self._special_combo.currentIndex()
        if vrata_index == len(vratha.special_vratha_map.keys()):#vratha_type in ['tithi','nakshatra','yogam']:
            _tithi_index = None if self._tithi_combo.currentIndex()==0 else self._tithi_combo.currentIndex()
            _nakshatra_index = None if self._nakshatra_combo.currentIndex()==0 else self._nakshatra_combo.currentIndex()
            _yogam_index = None if self._yogam_combo.currentIndex()==0 else self._yogam_combo.currentIndex()
            _tm_index = None if self._tm_combo.currentIndex()==0 else self._tm_combo.currentIndex()
            vdates = vratha.search(self.Place,_start_date, _end_date, _tithi_index, _nakshatra_index, _yogam_index, _tm_index)
        else:
            vratha_type = list(vratha.special_vratha_map.keys())[vrata_index]
            vdates = vratha.special_vratha_dates(self.Place, _start_date, _end_date, vratha_type)
        if len(vdates)>0:
            self._stats_label.setText(str(len(vdates))+' matching dates found')
            self._vratha_result_dates = vdates
            self._results_list.clear()
            for values in vdates:
                res = ''
                res += '-'.join(map(str,values[0]))
                if len(values)>1: res += ' / '+str(utils.to_dms(values[1]))
                if len(values)>2: res += ' / '+str(values[-1])
                self._results_list.addItem(res)
            self._results_list.setCurrentRow(0)
            self._accept_button.setEnabled(True)
        else:
            self._stats_label.setText('No matching dates found')
            self._results_list.clear()
    def _accept_and_close(self):
        selected_date_index = self._results_list.currentRow()
        #print('selected date index',selected_date_index)
        if selected_date_index !=-1:
            selected_item = self._vratha_result_dates[selected_date_index]
            #print('selected vdate',selected_item)
            selected_date = self._vratha_result_dates[selected_date_index][0]
            selected_time = self._vratha_result_dates[selected_date_index][1]
            self._selection_date_jd = utils.julian_day_number(selected_date, (0,0,0))
            #selected_time = drik.sunrise(self._selection_date_jd, self.Place)[0]
            self._selection_date_jd = utils.julian_day_number(selected_date, (selected_time,0,0))
            #print('select JD',self._selection_date_jd,selected_date,utils.to_dms(selected_time))
            y,m,d,fh = utils.jd_to_gregorian(self._selection_date_jd)
            #print(y,m,d,utils.to_dms(fh))
        self._accept_clicked = True
        self.accept()
    def _close_dialog(self):
        self._accept_clicked = False
        self.reject()
def show(current_date_jd,place:drik.Place,chart_title='Search for special days'):
    import sys
    def except_hook(cls, exception, traceback):
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    dlg = VrathaFinderDialog(current_date_jd,place,chart_title)
    ret = dlg.exec()
    print('exec return value',ret)
    return ret
if __name__ == "__main__":
    import sys
    def except_hook(cls, exception, traceback):
        print('exception called')
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    lang = 'ta'; utils.set_language(lang)
    App = QApplication(sys.argv)
    jd = utils.julian_day_number((2025,1,1), (8,34,0))
    place = drik.Place('Chennai,India',13.0878,80.2785,5.5)
    entry_type = 0
    show(jd, place)
    exit()
    chart = VrathaFinderDialog(jd,place)
    chart.show()
    sys.exit(App.exec())




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
    Module for Pancha Paksha Sastra UI
"""

import sys
from PyQt6 import QtCore
from PyQt6.QtGui import QIcon, QFont, QFontMetrics, QColor, QMovie
from PyQt6.QtWidgets import QApplication, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, \
                QLineEdit, QHBoxLayout,QLabel, QCompleter, QMessageBox, QPushButton, QComboBox, \
                QStyledItemDelegate, QProgressBar, QHeaderView
from PyQt6.QtCore import Qt, QDateTime, QRect, QTimer, QSize
from datetime import datetime
from jhora import utils, const
from jhora.panchanga import drik, pancha_paksha

IMAGE_PATH = const._IMAGES_PATH + const._sep
_WEEK_DAY_INDEX = 0; _PAKSHA_INDEX = 1; _DAYNIGHT_INDEX = 2
_NAK_BIRD_INDEX = 3; _NAK_ACTIVITY_INDEX = 4; _SUB_BIRD_INDEX = 5; _SUB_ACTIVITY_INDEX = 6
_DURATION_FACTOR = 7; _RELATION = 8; _POWER_FACTOR=9;_EFFECT = 10; _RATING=11;
_PADU_PAKSHI = 12; _BHARANA_PAKSHI = 13
_LAST_COL_FOR_READING = _BHARANA_PAKSHI#_RELATION
_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss"  # Custom format string

_FROM_DATE_DISP = 0; _TO_DATE_DISP=1; _DURATION_DISP=2; _MAIN_BIRD_DISP=3;
_MAIN_ACT_DISP = 4; _SUB_BIRD_DISP = 5; _SUB_ACT_DISP = 6; _RELATION_DISP = 7; _POWER_FACTOR_DISP = 8
_EFFECT_DISP = 9; _RATING_DISP = 10

_DEFAULT_ICON_SIZE = 16
#Pancha Pakshi for each star (birth if sukla paksha, birth if in krishna paksha)
#1:Vulture 2:Owl, 3:Crow, 4:Cock, 5:Peacock
pancha_pakshi_birds = ['vulture','owl','crow','cock','peacock']
pancha_pakshi_images = ['vulture.png','owl.png','crow.png','cock.png','peacock.png']
pancha_pakshi_activities = ['ruling','eating','walking','sleeping','dying']
pp_activity_background_colors = ['dark green','yellow','light green','orange','red']
pp_relations = ['enemy','same','friend']; pp_relation_colors = ['orange','yellow','light green']
pp_effect = ['very_bad','bad','average','good','very_good']
pp_effect_colors = ['red','orange','yellow','light green','dark green']
pancha_pakshi_activity_images = [p+'.png' for p in pancha_pakshi_activities]
def set_treewidget_font_size(tree, font_size):
    font = tree.font()
    font.setPointSizeF(font_size)  # Set the desired font size

    # Set the font for the QTreeWidget
    tree.setFont(font)

    # Set the font for the header
    tree.header().setFont(font)

    # Dynamically set the font for all items in the QTreeWidget
    def set_item_font(item):
        for column in range(tree.columnCount()):
            item.setFont(column, font)
        for child_index in range(item.childCount()):
            set_item_font(item.child(child_index))

    for row_index in range(tree.topLevelItemCount()):
        set_item_font(tree.topLevelItem(row_index))

def format_cell_value(cell):
    """Helper function to format cell values by handling new lines in strings and tuples."""
    if isinstance(cell, str):
        return cell.replace('\\n', '\n')
    elif isinstance(cell, tuple):
        return tuple(format_cell_value(item) for item in cell)
    else:
        return str(cell)
def add_text_above_parent_row(tree, parent_level_labels=None):
    # Create an intermediate item to hold the custom widget
    intermediate_item = QTreeWidgetItem()
    intermediate_item.setFlags(intermediate_item.flags() & ~Qt.ItemFlag.ItemIsSelectable)

    # Create a QWidget to hold the custom layout
    container = QWidget()
    layout = QHBoxLayout(container)
    layout.setAlignment(Qt.AlignmentFlag.AlignLeft)
    
    for (text,icon_path) in parent_level_labels:
        # Add QLabel to the layout
        label = QLabel(text)
        layout.addWidget(label)
        # Add icon if provided
        if icon_path:
            icon_label = QLabel()
            icon_label.setPixmap(QIcon(IMAGE_PATH+icon_path).pixmap(_DEFAULT_ICON_SIZE, _DEFAULT_ICON_SIZE))
            layout.addWidget(icon_label)
    
    layout.setContentsMargins(0, 0, 0, 0)  # Remove margins
    container.setLayout(layout)
    
    # Add the intermediate item with the custom widget to the tree
    tree.addTopLevelItem(intermediate_item)
    tree.setItemWidget(intermediate_item, 0, container)
        # Span the custom widget across all columns
    intermediate_item.setFirstColumnSpanned(True)

class CustomHeaderView(QHeaderView):
    def __init__(self, orientation, parent=None):
        super().__init__(orientation, parent)
        self.setDefaultAlignment(Qt.AlignmentFlag.AlignCenter)

    def paintSection(self, painter, rect, logicalIndex):
        painter.save()
        painter.setPen(Qt.GlobalColor.black)
        painter.setBrush(Qt.BrushStyle.SolidPattern)
        text = self.model().headerData(logicalIndex, self.orientation(), Qt.ItemDataRole.DisplayRole)
        if text:
            text_rect = QRect(rect)
            painter.drawText(text_rect, Qt.AlignmentFlag.AlignCenter, text)
        painter.restore()
class CustomDelegate(QStyledItemDelegate):
    def paint(self, painter, option, index):
        QStyledItemDelegate.paint(self, painter, option, index)

        if not index.parent().isValid():  # Only draw border for parent items
            parent_index = index
            parent_rect = option.widget.visualRect(parent_index)
            rect = parent_rect

            # Calculate the rectangle encompassing all children
            for i in range(option.widget.model().rowCount(parent_index)):
                child_index = option.widget.model().index(i, 0, parent_index)
                child_rect = option.widget.visualRect(child_index)
                rect = rect.united(child_rect)

            # Draw the border around the combined area of parent and children
            painter.save()
            painter.setPen(QColor(0, 0, 0))
            painter.drawRect(rect)
            painter.restore()

class PanchaPakshiSastraWidget(QWidget):
    def __init__(self, birth_bird_index=None, headers=None, parent_level_list=None, child_level_list=None, hide_headers=True,
                 parent_level_labels=None,tree_font_size=9,expand_one_row_at_a_time=False,synch_with_local_clock=False,
                 synch_timer_frequency_in_minutes=1):
        super().__init__()
        self.setWindowTitle(utils.resource_strings['pancha_pakshi_sastra_str'])
        self.setGeometry(50, 50, 800, 600)
        # Layout
        v_layout = QVBoxLayout()

        h_layout = QHBoxLayout()
        _dob_label = QLabel("Date:")
        h_layout.addWidget(_dob_label)
        self._dob_text = QLineEdit('')
        self._dob_text.setToolTip('Date in the format YYYY,MM,DD')
        _dob_label.setMaximumWidth(75)
        self._dob_text.setMaximumWidth(75)
        h_layout.addWidget(self._dob_text)
        _tob_label = QLabel("Time:")
        h_layout.addWidget(_tob_label)
        self._tob_text = QLineEdit('')
        self._tob_text.setToolTip('Enter time in the format HH:MM:SS if afternoon use 12+ hours')
        _tob_label.setMaximumWidth(75)
        self._tob_text.setMaximumWidth(75)
        current_date_str,current_time_str = datetime.now().strftime('%Y,%m,%d;%H:%M:%S').split(';')
        self._dob_text.setText(current_date_str)
        self._tob_text.setText(current_time_str)
        h_layout.addWidget(self._tob_text)

        self._place_name = ''
        self._place_text = QLineEdit(self._place_name)
        #_world_cities_list = utils.world_cities_list
        completer = QCompleter(utils.world_cities_dict.keys())
        completer.setCaseSensitivity(QtCore.Qt.CaseSensitivity.CaseInsensitive)
        self._place_text.setCompleter(completer)
        self._place_text.textChanged.connect(self._resize_place_text_size)
        self._place_text.editingFinished.connect(lambda : self._get_location(self._place_text.text()))
        self._place_text.setToolTip('Enter place of birth, country name')
        h_layout.addWidget(self._place_text)
        self._lat_label = QLabel("Latidude:")
        h_layout.addWidget(self._lat_label)
        self._lat_text = QLineEdit('')
        self._latitude = 0.0
        self._lat_text.setToolTip('Enter Latitude preferably exact at place of birth: Format: +/- xx.xxx')
        h_layout.addWidget(self._lat_text)
        self._long_label = QLabel("Longitude:")
        h_layout.addWidget(self._long_label)
        self._long_text = QLineEdit('')
        self._longitude = 0.0
        self._long_text.setToolTip('Enter Longitude preferably exact at place of birth. Format +/- xx.xxx')
        h_layout.addWidget(self._long_text)
        self._tz_label = QLabel("Time Zone:")
        h_layout.addWidget(self._tz_label)
        self._tz_text = QLineEdit('')
        self._time_zone = 0.0
        self._tz_text.setToolTip('Enter Time offset from GMT e.g. -5.5 or 4.5')
        h_layout.addWidget(self._tz_text)
        if const.use_internet_for_location_check:
            " Initialize with default place based on IP"
            loc = utils.get_place_from_user_ip_address()
            if len(loc)==4:
                self.place(loc[0],loc[1],loc[2],loc[3])
        _show_button = QPushButton('Show')
        _show_button.clicked.connect(self._calculate_results)
        h_layout.addWidget(_show_button)
        self.birth_bird_index = birth_bird_index if birth_bird_index is not None else 1
        self.bird_combo = QComboBox()
        self.bird_combo.addItems([utils.resource_strings[b+'_str'] for b in pancha_pakshi_birds])
        self.bird_combo.setCurrentIndex(self.birth_bird_index-1)
        h_layout.addWidget(self.bird_combo)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
        # Tree Widget
        self.tree = QTreeWidget()
        self._tree_font_size = tree_font_size
        set_treewidget_font_size(self.tree, tree_font_size)
        delegate = CustomDelegate()
        self.tree.setItemDelegate(delegate)
        # Connect to the itemExpanded signal
        if expand_one_row_at_a_time: self.tree.itemExpanded.connect(self.on_item_expanded)
        v_layout.addWidget(self.tree)
        self._display_sand_clock = False
        if synch_with_local_clock:
            # Set up the QTimer to update the row based on the current time
            self.timer = QTimer(self)
            self._search_datetime = None
            self.timer.timeout.connect(self.expand_row_with_datetime)
            self.timer.start(synch_timer_frequency_in_minutes*60000)
            self._display_sand_clock = True
        
        # Populate TreeWidget with data
        if headers and parent_level_list and child_level_list:
            self.populate_tree(birth_bird_index,headers,parent_level_list, child_level_list, hide_headers,
                               parent_level_labels=parent_level_labels,tree_font_size=self._tree_font_size)
    def restart_sandclock(self, timer_frequency):
        self.timer.stop()
        self.timer.start(timer_frequency)
    def place(self,place_name,latitude,longitude,timezone_hrs):
        """
            Set the place of birth
            @param - place_name - Specify with country code. e.g. Chennai, IN
            NOTE: Uses Nominatim to get the latitude and longitude
            An error message displayed if lat/long could not be found in which case enter lat/long manually.
            Also NOTE: calling latitude() or longitude() will replace the lat/long values added already
        """
        self._place_name = place_name
        self._latitude = latitude; self._longitude = longitude
        self._time_zone = timezone_hrs
        self._place_text.setText(self._place_name)
        self._lat_text.setText(str(self._latitude))
        self._long_text.setText(str(self._longitude))
        self._tz_text.setText(str(self._time_zone))
    def _resize_place_text_size(self):
        pt = self._place_text.text()
        f = QFont("",0)
        fm = QFontMetrics(f)
        pw = fm.boundingRect(pt).width()
        ph = fm.height()
        self._place_text.setFixedSize(pw,ph)
        self._place_text.adjustSize()
    def _get_location(self,place_name):
        result = utils.get_location(place_name)
        #print('RESULT',result)
        if result:
            self._place_name,self._latitude,self._longitude,self._time_zone = result
            self._place_text.setText(self._place_name)
            self._lat_text.setText(str(self._latitude))
            self._long_text.setText(str(self._longitude))
            self._tz_text.setText(str(self._time_zone))
        else:
            msg = place_name+" could not be found in OpenStreetMap.\nTry entering latitude and longitude manually.\nOr try entering nearest big city"
            print(msg)
            QMessageBox.about(self,"City not found",msg)
            self._lat_text.setText('')
            self._long_text.setText('')
        self._resize_place_text_size()
        
    def on_item_expanded(self, item):
        # Collapse all other top-level items
        for i in range(self.tree.topLevelItemCount()):
            top_item = self.tree.topLevelItem(i)
            if top_item != item:
                self.tree.collapseItem(top_item)
    def show_remove_sandclock(self,row_item,column_index,remove_clock=False):
        if remove_clock: 
            self.tree.setItemWidget(row_item, column_index, None)
            return
        sandclock_movie = QMovie(IMAGE_PATH+"sandclock.gif")
        sandclock_movie.setScaledSize(QSize(32, 32))  # Resize the GIF
        sandclock_label = QLabel()
        sandclock_label.setAlignment(Qt.AlignmentFlag.AlignLeft)
        sandclock_label.setMovie(sandclock_movie)
        sandclock_movie.start()
        
        widget = QWidget()
        layout = QHBoxLayout()
        layout.addWidget(sandclock_label)
        layout.setContentsMargins(0, 0, 0, 0)
        widget.setLayout(layout)
        
        self.tree.setItemWidget(row_item, column_index, widget)  # Set in the 6th column (index 5)
    def expand_row_with_datetime(self, search_datetime=None):
        if search_datetime is None:
            search_datetime = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            search_datetime = QDateTime.fromString(search_datetime, _DATE_FORMAT)
        _DEBUG_ = False
        if _DEBUG_: print('expand_row_with_datetime', search_datetime, 'isValid:', search_datetime.isValid())
        root = self.tree.invisibleRootItem()
        for i in range(root.childCount()):
            parent_item = root.child(i)
            from_datetime_string = parent_item.text(0)
            to_datetime_string = parent_item.text(1)
            if _DEBUG_: print('Parent item texts:', from_datetime_string, to_datetime_string)
            from_datetime = QDateTime.fromString(from_datetime_string, _DATE_FORMAT)
            to_datetime = QDateTime.fromString(to_datetime_string, _DATE_FORMAT)
            if _DEBUG_: print('Parsed datetimes:', from_datetime, to_datetime)
            if _DEBUG_: print(f"From isValid: {from_datetime.isValid()}, To isValid: {to_datetime.isValid()}")
    
            try:
                if _DEBUG_: print(f"Comparison: {from_datetime} <= {search_datetime} <= {to_datetime}")
                if from_datetime.isValid() and to_datetime.isValid() and from_datetime <= search_datetime <= to_datetime:
                    if _DEBUG_: print('Expanding parent', from_datetime, search_datetime, to_datetime)
                    self.tree.expandItem(parent_item)
                    #if self._display_sand_clock:
                    #    self.show_remove_sandclock(parent_item, _SUB_BIRD_DISP,remove_clock=False)
                    for j in range(parent_item.childCount()):
                        child_item = parent_item.child(j)
                        child_from_datetime_string = child_item.text(0)
                        child_to_datetime_string = child_item.text(1)
                        child_from_datetime = QDateTime.fromString(child_from_datetime_string, _DATE_FORMAT)
                        child_to_datetime = QDateTime.fromString(child_to_datetime_string, _DATE_FORMAT)
                        if _DEBUG_: print('Checking child:', child_from_datetime, child_to_datetime)
                        if _DEBUG_: print(f"Child Comparison: {child_from_datetime} <= {search_datetime} <= {child_to_datetime}")
                        if self._display_sand_clock:
                            self.show_remove_sandclock(child_item, _FROM_DATE_DISP,remove_clock=True)
                        if child_from_datetime.isValid() and child_to_datetime.isValid() and child_from_datetime <= search_datetime <= child_to_datetime:
                            if _DEBUG_: print('Found datetime', child_from_datetime, search_datetime, child_to_datetime)
                            #time_left = child_to_datetime.secsTo(search_datetime)//60
                            #print(search_datetime, child_to_datetime,'time left to move to next row',time_left,'minutes')
                            self.tree.expandItem(child_item)
                            self.tree.setCurrentItem(child_item)
                            if self._display_sand_clock:
                                self.show_remove_sandclock(child_item, _FROM_DATE_DISP,remove_clock=False)
                            #for col in range(self.tree.columnCount()):
                            #    child_item.setBackground(col, QColor('lightblue'))
                            break
                    break
            except Exception as e:
                print('Error processing search_datetime', search_datetime, 'Exception:', e)
    def populate_tree(self, birth_bird_index, headers, parent_level_list, child_level_list, hide_headers, 
                      search_datetime=None,parent_level_labels=None,tree_font_size=9):
        headers = [h.replace('\\n', '\n') for h in headers]
        self.tree.clear()
        self._tree_font_size = tree_font_size
        set_treewidget_font_size(self.tree, self._tree_font_size)
        self.bird_combo.setCurrentIndex(birth_bird_index-1)
        self.birth_bird_index = birth_bird_index
        # Set custom header view
        custom_header = CustomHeaderView(Qt.Orientation.Horizontal, self.tree)
        self.tree.setHeader(custom_header)
    
        if headers is not None:
            self.tree.setColumnCount(len(headers))
            self.tree.setHeaderLabels(headers)
        else:
            self.tree.setColumnCount(len(parent_level_list[0]))
        self.tree.setHeaderHidden(hide_headers)
    
        for parent_index, parent_row in enumerate(parent_level_list):
            if parent_level_labels:
                # Add custom text and icon above each parent row
                add_text_above_parent_row(self.tree, parent_level_labels[parent_index])
            parent_item = QTreeWidgetItem()
            self.set_item_icon_and_text(parent_item, parent_row)
            self.tree.addTopLevelItem(parent_item)
    
            for child_row in child_level_list[parent_index]:
                child_item = QTreeWidgetItem()
                # Check the type of each cell item
                processed_child_row = [format_cell_value(cell) for cell in child_row]
                self.set_item_icon_and_text(child_item, processed_child_row)
                parent_item.addChild(child_item)
    
                power_factor_progress_bar1 = QProgressBar()
                power_factor_progress_bar1.setValue(int(child_row[_POWER_FACTOR_DISP] * 100))  # Convert to percentage
                power_factor_progress_bar1.setStyleSheet("QProgressBar::chunk {background-color: lightgreen;}")
                power_factor_progress_bar1.setFixedWidth(60)
                self.tree.setItemWidget(child_item, _POWER_FACTOR_DISP, power_factor_progress_bar1)
                rating_progress_bar2 = QProgressBar()
                column_value = int(child_row[_RATING_DISP])  # Assuming column values are integers between 1 and 10
                rating_progress_bar2.setValue(column_value * 10)  # Convert to percentage
                rating_progress_bar2.setStyleSheet("QProgressBar::chunk {background-color: lightgreen;}")
                rating_progress_bar2.setFixedWidth(80)
                self.tree.setItemWidget(child_item, _RATING_DISP, rating_progress_bar2)
    
        self.tree.setColumnWidth(_RATING_DISP, 120)
        self.tree.itemExpanded.connect(self.resize_all_columns)
    
        if search_datetime:
            search_datetime = QDateTime.fromString(search_datetime, _DATE_FORMAT)
            self.expand_row_with_datetime(search_datetime)
        else:
            self._calculate_results()
    
        self.resize_all_columns()
        
    def set_item_icon_and_text(self, item, row):
        for column, cell_value in enumerate(row):
            if isinstance(cell_value,tuple):
                if len(cell_value)==3:
                    icon_path,text,color = cell_value
                    item.setBackground(column,QColor(color))
                else:
                    icon_path,text = cell_value
                if icon_path:
                    item.setIcon(column, QIcon(IMAGE_PATH+icon_path))
            else:
                text = cell_value
            item.setText(column, str(text))
    def resize_all_columns(self):
        """Resize columns of the tree widget to fit contents."""
        for column in range(self.tree.columnCount()):
            self.tree.resizeColumnToContents(column)
    def _calculate_results(self):
        from datetime import datetime
        year,month,day = self._dob_text.text().split(",")
        dob = (int(year),int(month),int(day))
        tob = tuple([int(x) for x in self._tob_text.text().split(':')])
        dob1 = datetime.strptime(self._dob_text.text(),"%Y,%m,%d")
        tob1 = datetime.strptime(self._tob_text.text(),"%H:%M:%S")
        sdate = datetime.combine(dob1.date(), tob1.time())
        search_datetime = sdate.strftime("%Y-%m-%d %H:%M:%S")
        place = drik.Place(self._place_text.text(),float(self._lat_text.text()),float(self._long_text.text()),float(self._tz_text.text()))
        bird_index = self.bird_combo.currentIndex()+1
        self.birth_bird_index = bird_index
        headers,top_level_list,child_level_list,parent_level_labels = pancha_paksha.construct_pancha_pakshi_information(dob, tob, place, nakshathra_bird_index=bird_index)
        self.populate_tree(bird_index,headers, top_level_list, child_level_list,hide_headers=False,
                           search_datetime=search_datetime,parent_level_labels=parent_level_labels,
                           tree_font_size=self._tree_font_size)        
if __name__ == "__main__":
    utils.set_language('ta')
    #_create_pancha_paksha_db(); exit()
    dob = drik.Date(1996,12,7); tob = (10,34,0); place = drik.Place('Chennai,India',13.0878,80.2785,5.5)
    jd = utils.julian_day_number(dob, tob)
    bs = pancha_paksha._get_birth_nakshathra(jd, place)
    paksha_index = pancha_paksha._get_paksha(jd, place)
    bird_index = pancha_paksha._get_birth_bird_from_nakshathra(bs,paksha_index)
    weekday_index = drik.vaara(jd)+1
    print('bird',bird_index,utils.resource_strings[pancha_pakshi_birds[bird_index-1]+'_str'])
    dt = datetime.now().timetuple()
    dob = dt[0:3]; tob=dt[3:6]
    headers,top_level_list,child_level_list,parent_level_labels = pancha_paksha.construct_pancha_pakshi_information(dob, tob, place, nakshathra_bird_index=bird_index)
    app = QApplication(sys.argv)
    demo = PanchaPakshiSastraWidget(birth_bird_index=bird_index,headers=headers,parent_level_list=top_level_list,child_level_list=child_level_list,
                          hide_headers=False,tree_font_size=6.7,synch_with_local_clock=True,expand_one_row_at_a_time=False)
    demo.show()
    sys.exit(app.exec())



import sys, os
import datetime
from PyQt6.QtWidgets import (QApplication, QWidget, QGridLayout, QPushButton, QCompleter, QMessageBox,
                            QLineEdit, QVBoxLayout, QHBoxLayout, QLabel, QComboBox, QDialog)
from PyQt6.QtCore import Qt, QRect, pyqtSignal, QPoint, pyqtSlot
from jhora.panchanga import drik, pancha_paksha
from jhora import utils, const
from PyQt6.QtGui import QFont, QFontMetrics, QPainter, QColor, QPen, QPixmap, QIcon
from jhora.ui.panchangam import PanchangaInfoDialog

_SHOW_MUHURTHA_OR_SHUBHA_HORA = 1 # 0=Muhurtha 1=Shubha Hora
_sunrise_icon = os.path.abspath(const._IMAGES_PATH) + const._sep + "sunrise1.png"
_sunset_icon = os.path.abspath(const._IMAGES_PATH) + const._sep + "sunset1.png"
_pournami_icon = os.path.abspath(const._IMAGES_PATH) + const._sep + "pournami.png"
_amavasai_icon = os.path.abspath(const._IMAGES_PATH) + const._sep + "amavasai.png"
_shukla_paksha_icon = os.path.abspath(const._IMAGES_PATH) + const._sep + "shukla_paksha.png"
_krishna_paksha_icon = os.path.abspath(const._IMAGES_PATH) + const._sep + "krishna_paksha.png"
_info_label1_font_size = 6#8
_info_label2_font_size = 6 if _SHOW_MUHURTHA_OR_SHUBHA_HORA==1 else 5
_info_label3_font_size =6#8
_ICON_SIZE = 12
_top_left_font_size = 9; _top_right_font_size = 9
_bottom_left_font_size = 9; _bottom_right_font_size = 9
_center_left_font_size = 12; _center_right_font_size = 16
_calendar_cell_width = 100;_calendar_cell_height = 80 
_label_offset_x = 2; _label_offset_y = 1
#_info_label1_font_size = 6
_info_label1_width = 325
_HEADER_CELL_HEIGHT = 20; _HEADER_COLOR='green'; _HEADER_LABEL_COLOR = 'maroon'
_KEY_COLOR = 'brown'; _VALUE_COLOR = 'blue'
_KEY_LENGTH=50; _VALUE_LENGTH=50; _HEADER_LENGTH=100
_HEADER_FORMAT_ = '<b><span style="color:'+_HEADER_COLOR+';">{:.'+str(_HEADER_LENGTH)+'}</span></b><br>'
_KEY_VALUE_FORMAT_ = '<span style="color:'+_KEY_COLOR+';">{:.'+str(_KEY_LENGTH)+'}</span><span style="color:'+\
        _VALUE_COLOR+';">{:.'+str(_VALUE_LENGTH)+'}</span><br>'
previous_month_color = "orange"; default_color = "lightyellow"
input_day_color = "lightgreen"; next_month_color = "lightblue"
_top_left_label_color = 'magenta'; _top_right_label_color = 'blue'
_middle_left_label_color = 'red'; _middle_right_label_color = 'black'
_bottom_left_label_color = 'green'; _bottom_right_label_color = 'blue'
_cell_border_line_color = 'brown'; _cell_border_line_thickness = 1

class CustomLabel(QLabel):
    clicked = pyqtSignal()  # Signal for click event

    def __init__(self, parent=None):
        super().__init__(parent)
        self.texts = {}
        self.font_sizes = {
            "top_left": _top_left_font_size, "top_right": _top_right_font_size,
            "middle_left":_center_left_font_size,"middle_right": _center_right_font_size,
            "bottom_left": _bottom_left_font_size, "bottom_right": _bottom_right_font_size ,
            "center":_center_right_font_size
        }
        self.colors = {
            "top_left": QColor(_top_left_label_color), "top_right": QColor(_top_right_label_color),
            "middle_left": QColor(_middle_left_label_color),"middle_right": QColor(_middle_right_label_color),
            "bottom_left": QColor(_bottom_left_label_color), "bottom_right": QColor(_bottom_right_label_color),
            "center":QColor(_middle_right_label_color),
        }
        self.setStyleSheet("border: "+str(_cell_border_line_thickness)+"px solid "+_cell_border_line_color+";")
    def set_texts(self, text_pos):
        """Set texts at specified positions."""
        self.texts = text_pos
        self.update()

    def paintEvent(self, event):
        try:
            super().paintEvent(event)
            painter = QPainter(self)
            painter.setRenderHint(QPainter.RenderHint.Antialiasing)
            painter.setPen(QColor(Qt.GlobalColor.black))
            for pos, value in self.texts.items():
                alignment = self.get_alignment(pos)
                cell_width = self.width() // 2
                cell_height = self.height() // 3
                cell_x = (0 if 'left' in pos else self.width() // 2)
                cell_y = (0 if 'top' in pos else self.height() // 3 if 'middle' in pos else 2 * self.height() // 3)
                cell_y = self.height()//2 if 'center' in pos else cell_y
                # Adjust the size of the bounding rect to ensure it fully fits within the grid cell
                text_rect = QRect(cell_x, cell_y, cell_width, cell_height)
                rect = QRect(cell_x+_label_offset_x,cell_y+_label_offset_y,
                             text_rect.width()-_label_offset_x,text_rect.height()-_label_offset_y
                             )
                # Set font size based on position
                font = painter.font()
                font_size = self.font_sizes.get(pos, 9)
                pen_color = self.colors.get(pos,QColor('black')); painter.setPen(pen_color)
                font.setPointSize(font_size); font.setBold(True)
                painter.setFont(font)
                # Check if the value is a tuple (icon_path, text)
                if isinstance(value, tuple):
                    icon_path, text = value
                    icon = QPixmap(icon_path).scaled(_ICON_SIZE, _ICON_SIZE, Qt.AspectRatioMode.KeepAspectRatio, Qt.TransformationMode.SmoothTransformation)
                    icon_rect = QRect(cell_x, cell_y, _ICON_SIZE, _ICON_SIZE)
                    painter.drawPixmap(icon_rect, icon)
                    rect.setLeft(icon_rect.right())  # Add some space between icon and text
                else:
                    text = value
                painter.drawText(rect, int(alignment), str(text))
                painter.setPen(QPen())
            painter.end()
        except Exception as e:
            print(f"CustomLabel paintEvent - An error occurred: {e}")
    
    def get_alignment(self, pos):
        if "top" in pos:
            vertical = Qt.AlignmentFlag.AlignTop
        elif "bottom" in pos:
            vertical = Qt.AlignmentFlag.AlignBottom
        else:
            vertical = Qt.AlignmentFlag.AlignVCenter
            
        if "left" in pos:
            horizontal = Qt.AlignmentFlag.AlignLeft
        elif "right" in pos:
            horizontal = Qt.AlignmentFlag.AlignRight
        else:
            horizontal = Qt.AlignmentFlag.AlignHCenter
            
        return horizontal | vertical

    def mousePressEvent(self, event):
        self.clicked.emit()  # Emit signal when label is clicked

class VedicCalendar(QWidget):
    def __init__(self,start_date:drik.Date=None,place:drik.Place=None, language='ta',use_purnimanta_system=None,
                 use_world_city_database=const.check_database_for_world_cities,
                 use_internet_for_location_check=const.use_internet_for_location_check):
        """
            @param use_purnimanta_system: None => Solar Calendar, False=>Amantha, True=>Purnimantha 
        """
        super().__init__()
        self._language = language
        self.use_world_city_database = use_world_city_database
        utils.set_language(const.available_languages[language])
        utils.use_database_for_world_cities(self.use_world_city_database)
        self.use_internet_for_location_check = use_internet_for_location_check
        self._use_purnimanta_system = use_purnimanta_system
        self.res = utils.resource_strings
        self.start_date = start_date; self.start_place=place
        self.selected_cell = None; self.previous_month_cell=None; self.next_month_cell=None
        self.setWindowTitle(self.res['calendar_str']+' '+const._APP_VERSION)
        self.setWindowIcon(QIcon(const._IMAGE_ICON_PATH))
        self.initUI()

    def initUI(self):
        main_layout = QVBoxLayout()
        input_layout = QHBoxLayout()
        if self.start_date is None:
            current_date_str = datetime.datetime.now().strftime('%Y,%m,%d')
        else:
            year,month,day = self.start_date
            current_date_str = str(year)+','+str(month)+','+str(day)
        self.date_text = QLineEdit(self)
        self.date_text.setText(current_date_str)
        self.date_text.editingFinished.connect(self.computeCalendar)
        if self.start_place is None and self.use_internet_for_location_check:
            loc = utils.get_place_from_user_ip_address()
            print('loc from IP address',loc)
            if len(loc)==4:
                print('setting values from loc')
                self.start_place = drik.Place(loc[0],loc[1],loc[2],loc[3])
        self._place_text = QLineEdit(self)
        completer = QCompleter(utils.world_cities_dict.keys())
        completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        self._place_text.setCompleter(completer)
        self._place_text.textChanged.connect(self._resize_place_text_size)
        if self.use_internet_for_location_check: self._place_text.editingFinished.connect(lambda: self._get_location(self._place_text.text()))
        self._place_text.setToolTip('Enter place of birth, country name')
        self._lat_text = QLineEdit(self)
        self._long_text = QLineEdit(self)
        self._tz_text = QLineEdit(self)
        if isinstance(self.start_place,drik.Place):
            self._place_text.setText(self.start_place.Place)
            self._lat_text.setText(str(self.start_place.latitude))
            self._long_text.setText(str(self.start_place.longitude))
            self._tz_text.setText(str(self.start_place.timezone))
        self._dob_label = QLabel('Date:')
        input_layout.addWidget(self._dob_label)
        input_layout.addWidget(self.date_text)
        self._place_label = QLabel('Place Name:')
        input_layout.addWidget(self._place_label)
        input_layout.addWidget(self._place_text)
        self._lat_label = QLabel('Latitude:')
        input_layout.addWidget(self._lat_label)
        input_layout.addWidget(self._lat_text)
        self._long_label = QLabel('Longitude:')
        input_layout.addWidget(self._long_label)
        input_layout.addWidget(self._long_text)
        self._tz_label = QLabel('TimeZone Hours:')
        input_layout.addWidget(self._tz_label)
        input_layout.addWidget(self._tz_text)
        self._lang_combo = QComboBox()
        self._lang_combo.addItems(const.available_languages.keys())
        self._lang_combo.setCurrentText(self._language)
        self._lang_combo.activated.connect(self._change_language)
        self._lang_combo.setToolTip('Choose language for display')
        input_layout.addWidget(self._lang_combo)
        self._calendar_combo = QComboBox()
        _calendar_list = [self.res[s]+' '+self.res['calendar_str'] for s in ['solar_str','amantha_str','purnimantha_str']]
        self._calendar_combo.addItems(_calendar_list)
        self._calendar_combo.activated.connect(self._change_language)
        _calendar_index = 0 if self._use_purnimanta_system is None else (2 if self._use_purnimanta_system else 1)
        self._calendar_combo.setCurrentIndex(_calendar_index)
        input_layout.addWidget(self._calendar_combo)
        self.compute_btn = QPushButton(self.res['calendar_str'], self)
        self.compute_btn.clicked.connect(self.computeCalendar)
        input_layout.addWidget(self.compute_btn)

        main_layout.addLayout(input_layout)

        h_layout = QHBoxLayout()
        self._info_label1 = QLabel("Information:")
        self._info_label1.setFixedWidth(_info_label1_width)
        self._info_label1.setStyleSheet("border: 1px solid black; font-size: "+str(_info_label1_font_size)+" pt")
        # Use setTextFormat and setTextInteractionFlags to make the link clickable
        self._info_label1.setTextFormat(Qt.TextFormat.RichText)
        self._info_label1.setTextInteractionFlags(Qt.TextInteractionFlag.TextBrowserInteraction)
        self._info_label1.setOpenExternalLinks(False)  # Disable opening links in the default browser
        h_layout.addWidget(self._info_label1)
        v_layout = QVBoxLayout()
        self.header_label = QLabel('')
        self.header_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.header_label.setStyleSheet("border: 1px solid black; font-weight: bold; font-size: "+str(12)+"pt")
        v_layout.setSpacing(0); v_layout.setContentsMargins(0,0,0,0)
        v_layout.addWidget(self.header_label)
        self.grid_layout = QGridLayout()
        self.grid_layout.setSpacing(0)
        self.grid_layout.setContentsMargins(0, 0, 0, 0)
        self.cells = []; self.jd = []

        headers = ['<b><span style="color:'+_HEADER_COLOR+';">'+str(h)+'</span></b>' for h in utils.DAYS_SHORT_NAMES]
        self.day_labels = ['' for _ in utils.DAYS_SHORT_NAMES]
        for col, header in enumerate(headers):
            self.day_labels[col] = QLabel(header)
            self.day_labels[col].setFixedHeight(_HEADER_CELL_HEIGHT)  # Set a fixed height for the header
            self.day_labels[col].setAlignment(Qt.AlignmentFlag.AlignCenter)  # Center align the text
            _cell_style = "border: "+str(_cell_border_line_thickness)+"px solid "+_cell_border_line_color+";"
            self.day_labels[col].setStyleSheet(_cell_style)
            self.grid_layout.addWidget(self.day_labels[col], 0, col)

        for row in range(1, 8):
            week = []; jd_week = []
            for col in range(7):
                cell = CustomLabel(self)
                cell.setMinimumSize(_calendar_cell_width, _calendar_cell_height)
                cell.setVisible(False)
                cell.clicked.connect(lambda r=row, c=col: self.cell_clicked(r, c))
                self.grid_layout.addWidget(cell, row, col)
                week.append(cell); jd_week.append(None)
            self.cells.append(week); self.jd.append(jd_week)

        v_layout.addLayout(self.grid_layout)
        h_layout.addLayout(v_layout)
        main_layout.addLayout(h_layout)
        self.move(50,50)
        self.setLayout(main_layout)
        self.computeCalendar()
    
    @pyqtSlot(str)
    def _on_show_more_link_clicked(self, link,jd, place):
        if link == "show_more":
            try:
                _info_dialog = QDialog(self) 
                _info_dialog.setWindowTitle(self.res['panchangam_str'])
                dialog_layout = QVBoxLayout()
                panchanga_widget = PanchangaInfoDialog(language=self._language,jd=jd,place=place)
                dialog_layout.addWidget(panchanga_widget)
                _info_dialog.setLayout(dialog_layout)
                _info_dialog.exec()
            except Exception as e:
                print(f"An error occurred: {e}")
    def _change_language(self):
        self._language = self._lang_combo.currentText()
        _calendar_index = self._calendar_combo.currentIndex()
        if _calendar_index ==0:
            self._use_purnimanta_system = None
        elif _calendar_index ==1:
            self._use_purnimanta_system = False
        else: #_calendar_index ==2:
            self._use_purnimanta_system = True
        
        utils.set_language(const.available_languages[self._language])
        self.res = utils.resource_strings
        self.setWindowTitle(self.res['calendar_str']+' '+const._APP_VERSION)
        self.computeCalendar()
        
    def _reshade_cells(self):
        for row in range(7):
            for col in range(7):
                cell = self.cells[row][col]
                if cell.isVisible():
                    _cell_style = "border: "+str(_cell_border_line_thickness)+"px solid "+_cell_border_line_color+"; "
                    if (row,col)==self.selected_cell:
                        _cell_style +=  f"background-color: {input_day_color}"
                    elif (row,col)==self.previous_month_cell:
                        _cell_style +=  f"background-color: {previous_month_color}"
                    elif (row,col)==self.next_month_cell:
                        _cell_style +=  f"background-color: {next_month_color}"
                    else:
                        _cell_style +=  f"background-color: {default_color}"
                    cell.setStyleSheet(_cell_style)
    def _resize_place_text_size(self):
        pt = self._place_text.text()
        f = QFont("", 0)
        fm = QFontMetrics(f)
        pw = fm.boundingRect(pt).width()
        ph = fm.height()
        self._place_text.setFixedSize(pw, ph)
        self._place_text.adjustSize()

    def _get_location(self, place_name):
        #print('get_location',place_name)
        result = utils.get_location(place_name)
        #print('RESULT', result)
        if result:
            self._place_name, self._latitude, self._longitude, self._time_zone = result
            self._place_text.setText(self._place_name)
            self._lat_text.setText(str(self._latitude))
            self._long_text.setText(str(self._longitude))
            self._tz_text.setText(str(self._time_zone))
            #print(self._place_name, self._latitude, self._longitude, self._time_zone)
        else:
            msg = place_name + " could not be found in OpenStreetMap.\nTry entering latitude and longitude manually.\nOr try entering nearest big city"
            print(msg)
            QMessageBox.about(self, "City not found", msg)
            self._lat_text.setText('')
            self._long_text.setText('')
        self._resize_place_text_size()

    def _get_days_panchanga_info(self,row,col):
        jd = self.jd[row][col]
        if jd is not None:
            place = self.start_place; y,m,d,_ = utils.jd_to_gregorian(jd); date_in = drik.Date(y,m,d)
            _tit = drik.tithi(jd, place)[0]
            _tithi_icon = _pournami_icon if _tit==15 else (_amavasai_icon if _tit==30 else '' )
            _tithi = utils.TITHI_SHORT_LIST[_tit-1]
            _paksha = 0 if _tit<=15 else  1
            kp_icon = _shukla_paksha_icon if _paksha==0 else  _krishna_paksha_icon
            _srise = utils.to_dms(drik.sunrise(jd, place)[0],round_to_minutes=True).strip()
            _sset = utils.to_dms(drik.sunset(jd, place)[0],round_to_minutes=True).strip()
            _nak = utils.NAKSHATRA_SHORT_LIST[drik.nakshatra(jd, place)[0]-1]
            _lang = const.available_languages[self._language]
            if self._use_purnimanta_system is None:
                tm,td = drik.tamil_solar_month_and_date(date_in, place)
                spl_month_text = utils.MONTH_LIST[tm]+' '+str(td)
                _samvatsara = drik.samvatsara(date_in, place, zodiac=0)
                year_str = utils.YEAR_LIST[_samvatsara]
            else:
                maasam_no,adhik_maasa,nija_maasa,td,_lunar_year = drik.lunar_month_date(jd,place,
                                                        use_purnimanta_system=self._use_purnimanta_system)
                adhik_maasa_str = ''; 
                if adhik_maasa:
                    adhik_maasa_str = self.res['adhika_maasa_str']
                """ Check if current month is Nija Maasa """
                nija_month_str = ''
                if nija_maasa:
                    nija_month_str = self.res['nija_month_str']
                spl_month_text = utils.MONTH_LIST[maasam_no-1]+' '+ adhik_maasa_str+nija_month_str+' '+str(td)
                year_str = utils.YEAR_LIST[_lunar_year]
            _panchanga_dict = {'top_left': (kp_icon,_tithi), 'middle_left':(_tithi_icon,str(td)),
                               'middle_right':('',str(d)), 'bottom_left': _nak,
                               'top_right': (_sunrise_icon,_srise), 'bottom_right':(_sunset_icon,_sset)}
            header_text = year_str+' '+str(y)+' '+ utils.MONTH_LIST_EN[m-1]+' '+str(d)+' '+utils.DAYS_LIST[drik.vaara(jd)]+' '+\
                          spl_month_text+' '+ utils.PAKSHA_LIST[_paksha]+' ' + \
                          utils.TITHI_LIST[_tit-1]
            return _panchanga_dict, header_text
    def _update_resources(self):
        msgs = self.res
        self._dob_label.setText(msgs['date_of_birth_str'])
        self._dob_label.setToolTip(msgs['dob_tooltip_str'])
        self._place_label.setText(msgs['place_str'])
        self._place_label.setToolTip(msgs['place_tooltip_str'])
        self._lat_label.setText(msgs['latitude_str'])
        self._lat_label.setToolTip(msgs['latitude_tooltip_str'])
        self._long_label.setText(msgs['longitude_str'])
        self._long_label.setToolTip(msgs['longitude_tooltip_str'])
        self._tz_label.setText(msgs['timezone_offset_str'])
        self._tz_label.setToolTip(msgs['timezone_tooltip_str'])
        self.compute_btn.setText(msgs['calendar_str'])
        _calendar_list = [self.res[s]+' '+self.res['calendar_str'] for s in ['solar_str','amantha_str','purnimantha_str']]
        _cal_index = self._calendar_combo.currentIndex()
        self._calendar_combo.clear()
        self._calendar_combo.addItems(_calendar_list)
        self._calendar_combo.setCurrentIndex(_cal_index)
    def computeCalendar(self):
        try:
            self._update_resources()
            date_str = self.date_text.text()
            place_name = self._place_text.text()
            latitude = float(self._lat_text.text())
            longitude = float(self._long_text.text())
            timezone = float(self._tz_text.text())
            self.start_place = drik.Place(place_name, latitude, longitude, timezone)
            year, month, day = utils.get_year_month_day_from_date_format(date_str)#map(int,date_str.split(","))#
            selected_date = drik.Date(year,month,day)
            _jd = utils.julian_day_number((year, month, 1), (10, 0, 0))
            sunrise_hours = drik.sunrise(_jd,self.start_place)[0]
            _jd = utils.julian_day_number((year, month, 1), (sunrise_hours, 0, 0))
            first_day = drik.Date(year, month, 1)
            previous_month_end = utils.previous_panchanga_day(first_day, minus_days=1)
            if month == 12:
                last_day = 31
                next_month_start = drik.Date(year+1,1,1)
            else:
                last_day = utils.next_panchanga_day(drik.Date(year,(month+1)%13,1), -1)
                next_month_start = utils.next_panchanga_day(last_day, add_days=1)
                last_day = last_day.day
            _jd -= 1; current_date = previous_month_end
            start_day = drik.vaara(_jd)
            reached_end_of_month = False
            [self.cells[row][col].setVisible(False) for col in range(7) for row in range(7)]
            for row in range(7):
                for col in range(7):
                    cell = self.cells[row][col]
                    cell.clear()
                    if reached_end_of_month:
                        break
                    if row * 7 + col >= start_day:
                        _year,_month,_day,_ = utils.jd_to_gregorian(_jd)
                        sunrise_hours = drik.sunrise(_jd,self.start_place)[0]
                        _jd = utils.julian_day_number((_year, _month, _day), (sunrise_hours, 0, 0))
                        self.jd[row][col]=_jd
                        cell.setVisible(True)
                        panchanga_dict,_ = self._get_days_panchanga_info(row,col)
                        if panchanga_dict is not None: cell.set_texts(panchanga_dict)
                        _cell_style = "border: "+str(_cell_border_line_thickness)+"px solid "+_cell_border_line_color+"; "
                        if current_date == selected_date:
                            _cell_style += f"background-color: {input_day_color}"
                            self.selected_cell = (row+1,col)
                        elif current_date == previous_month_end:
                            _cell_style += f"background-color: {previous_month_color}"
                            self.previous_month_cell = (row,col)
                        elif current_date == next_month_start:
                            _cell_style += f"background-color: {next_month_color}"
                            self.next_month_cell = (row,col)
                        else:
                            _cell_style += f"background-color: {default_color}"
                        cell.setStyleSheet(_cell_style)
                            
                        reached_end_of_month = (current_date == next_month_start)
                        current_date = utils.next_panchanga_day(current_date, add_days=1)
                        _jd += 1
            if self.selected_cell is not None:
                self.cell_clicked(self.selected_cell[0], self.selected_cell[1])
        except Exception as e:
            print(f"An error occurred: {e}")

    def cell_clicked(self,row,col):
        try:
            self.selected_cell = (row-1,col)
            jd = self.jd[row-1][col]; place = self.start_place
            self._reshade_cells()
            sep_str = '\n'
            info_list = self._fill_information_label1(jd,place).split(sep_str)
            _,header_text = self._get_days_panchanga_info(row-1,col)
            #print(jd,utils.jd_to_gregorian(jd),header_text)
            self.header_label.setText('<b><span style="color:'+_HEADER_LABEL_COLOR+';">'+str(header_text)+'</span></b>')
            for c,day in enumerate(utils.DAYS_SHORT_NAMES):
                _cell_style = "border: "+str(_cell_border_line_thickness)+"px solid "+_cell_border_line_color+";"
                self.day_labels[c].setStyleSheet(_cell_style)
                self.day_labels[c].setText('<b><span style="color:'+_HEADER_COLOR+';">'+str(day)+'</span></b>')
            font = QFont(); font.setPointSize(_info_label1_font_size); self._info_label1.setFont(font)
            self._info_label1.setText(sep_str.join(info_list[:]))
            y,m,d,_ = utils.jd_to_gregorian(self.jd[row-1][col])
            self.date_text.setText(str(y)+','+str(m)+','+str(d))
            if self.selected_cell == self.previous_month_cell or self.selected_cell==self.next_month_cell:
                self.computeCalendar()
        except Exception as e:
            tb = sys.exc_info()[2]
            print(f"An error occurred: {e}",'line number',tb.tb_lineno)
    def _fill_information_label1(self,jd,place,show_more_link=True):
        try:
            info_str = ''; format_str = _KEY_VALUE_FORMAT_
            info_str = PanchangaInfoDialog._fill_information_label1(self,show_more_link=False, jd=jd, place=place,
                                                                    ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE)
            if show_more_link:
                info_str += format_str.format('<a href="show_more">Show more</a>','')
                try:
                    self._info_label1.linkActivated.disconnect()
                except TypeError:
                    pass
                self._info_label1.linkActivated.connect(lambda link: self._on_show_more_link_clicked(link, jd, place))
            return info_str
        except Exception as e:
            tb = sys.exc_info()[2]
            print(f"An error occurred: {e}",'line number',tb.tb_lineno)
if __name__ == '__main__':
    try:
        app = QApplication(sys.argv)
        lang = 'Tamil'
        #start_date = (-3101,1,22); start_place = drik.Place('Ujjain,India',23.18,75.77,5.5)
        start_date = None; start_place = None
        ex = VedicCalendar(start_date=start_date,place=start_place,language=lang,use_purnimanta_system=False)
        ex.show()
        sys.exit(app.exec())
    except Exception as e:
        print(f"An error occurred: {e}")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from PyQt6.QtWidgets import QLineEdit, QApplication, QLabel, QHBoxLayout, QVBoxLayout, QPushButton,\
                            QRadioButton, QDialog, QCheckBox, QComboBox
from PyQt6.QtCore import Qt
from jhora import utils, const
class WidgetDialog(QDialog):
    """
        TODO: DONOT USE THIS CLASS YET
        Passed WIDGETS ARE DELETED AT CLOSE
    """
    """
        Widget Dialog with buttons and title
        Displays the widget, buttons and title
    """
    def __init__(self, title='',h_widgets=[],fit_to_widget_contents=False):
        super().__init__()
        self._fit_to_widget_contents = fit_to_widget_contents
        self.res = utils.resource_strings
        self._title = title
        self._h_widgets = h_widgets
        self.setWindowFlags(Qt.WindowType.Dialog | Qt.WindowType.WindowMinimizeButtonHint 
                            | Qt.WindowType.WindowMaximizeButtonHint|Qt.WindowType.WindowCloseButtonHint)
        self.create_ui()
    def create_ui(self):
        v_layout = QVBoxLayout()
        h_layout = QHBoxLayout()
        for widget in self._h_widgets:
            h_layout.addWidget(widget)
        v_layout.addLayout(h_layout)
        self.setWindowTitle(self._title)
        self.adjustSize()
        self.setMinimumSize(self.sizeHint()) if self._fit_to_widget_contents else self.setMinimumSize(700,650)
        self.setLayout(v_layout)
    def _close_dialog(self):
        self.reject()
    def closeEvent(self, *args, **kwargs):
        QApplication.restoreOverrideCursor()
        self.accept()
        return QDialog.closeEvent(self, *args, **kwargs)
class InfoDialog(QDialog):
    """
        Information Dialog with buttons and title
        Contains configured Label, buttons and title
    """
    def __init__(self, title='',info_text='', button_texts=[]):
        super().__init__()
        self.res = utils.resource_strings
        self._title = title
        self._info_text = info_text
        self._button_texts = button_texts if len(button_texts)>0 else [self.res['accept_str']]
        self.create_ui()
    def create_ui(self):
        v_layout = QVBoxLayout()
        _label = QLabel(self._info_text)
        v_layout.addWidget(_label)
        h_layout = QHBoxLayout()
        for btn_text in self._button_texts:
            btn = QPushButton(btn_text)
            h_layout.addWidget(btn)
            btn.clicked.connect(self._close_dialog)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
        self.setWindowTitle(self._title)
    def _close_dialog(self):
        self.reject()
    def closeEvent(self, *args, **kwargs):
        self._option_string = ''
        QApplication.restoreOverrideCursor()
        return QDialog.closeEvent(self, *args, **kwargs)

class OptionDialog(QDialog):
    """
        General Options Dialog
    """
    def __init__(self, title='', option_label='', options_list=None, multi_selection=None, default_options=None,
                 options_orientation=0):
        super().__init__()
        self.res = utils.resource_strings
        self._title = 'Options' if title.strip() == '' else title
        self._option_label = self.res['select_one_of_options_from_str'] if option_label.strip() == '' else option_label
        self._options_list = options_list if options_list is not None else []
        self._multi_selection = multi_selection
        self._default_options = default_options if default_options is not None else [0]
        self._options_orientation = options_orientation
        self.create_ui()

    def create_ui(self):
        v_layout = QVBoxLayout()
        _label = QLabel(self._option_label)
        v_layout.addWidget(_label)
        
        self._options = []
        
        if len(self._options_list) == 0: 
            return
        
        if self._multi_selection is None:
            if isinstance(self._options_list[0], list):
                if self._options_orientation == 1:  # Vertical
                    for sublist in self._options_list:
                        combobox = QComboBox()
                        combobox.addItems(sublist)
                        self._options.append(combobox)
                        v_layout.addWidget(combobox)
                else:  # Horizontal
                    h_layout = QHBoxLayout()
                    for sublist in self._options_list:
                        combobox = QComboBox()
                        combobox.addItems(sublist)
                        self._options.append(combobox)
                        h_layout.addWidget(combobox)
                    v_layout.addLayout(h_layout)
            else:
                combobox = QComboBox()
                combobox.addItems(self._options_list)
                self._options.append(combobox)
                v_layout.addWidget(combobox)
        elif self._multi_selection:
            for mc in range(len(self._options_list)):
                _caption = self._options_list[mc]
                self._options.append(QCheckBox(_caption))
                if mc in self._default_options: 
                    self._options[mc].setChecked(True)
                v_layout.addWidget(self._options[mc])
        else:
            for mc in range(len(self._options_list)):
                _caption = self._options_list[mc]
                print(mc,_caption,self._default_options)
                self._options.append(QRadioButton(_caption))
                if mc in [self._default_options]:
                    self._options[mc].setChecked(True)
                v_layout.addWidget(self._options[mc])
        
        h_layout = QHBoxLayout()
        self._accept_button = QPushButton(self.res['accept_str'])
        self._accept_button.clicked.connect(self._accept_button_clicked)
        h_layout.addWidget(self._accept_button)
        self._cancel_button = QPushButton(self.res['cancel_str'])
        self._cancel_button.clicked.connect(self._cancel_button_clicked)
        h_layout.addWidget(self._cancel_button)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
        self.setWindowTitle(self._title)
    
    def _accept_button_clicked(self):
        self._accept_clicked = True
        if self._multi_selection is None:
            self._option_string = [cb.currentText() for cb in self._options]
            self._option_index = [cb.currentIndex() + 1 for cb in self._options]
        elif self._multi_selection:
            self._option_string = []
            self._option_index = []
            for mc, rb in enumerate(self._options):
                if rb.isChecked():
                    self._option_string.append(rb.text())
                    self._option_index.append(mc + 1)
        else:
            for mc, rb in enumerate(self._options):
                if rb.isChecked():
                    self._option_string = rb.text()
                    self._option_index = mc + 1
        #print(self._option_index, self._option_string)
        self.accept()

    def _cancel_button_clicked(self):
        self._accept_clicked = False
        self._option_string = [] if self._multi_selection is not None else ''
        self._option_index = [] if self._multi_selection is not None else ''
        self.reject()
    
    def closeEvent(self, *args, **kwargs):
        self._option_string = ''
        QApplication.restoreOverrideCursor()
        return QDialog.closeEvent(self, *args, **kwargs)

if __name__ == "__main__":
    import sys
    utils.set_language('ta')
    def except_hook(cls, exception, traceback):
        print('exception called')
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    App = QApplication(sys.argv)
    """
    dlg = InfoDialog(title=utils.resource_strings['saham_str'],button_texts=[utils.resource_strings['accept_str']],
                     info_text=utils.resource_strings['saptha_shalaka_str']+'<br>'+
                     utils.resource_strings['prachanda_str'])
    dlg.show()
    sys.exit(App.exec())
    """
    #"""
    _title_list = ['solar','amantha','purnimantha']
    _title = '/'.join([utils.resource_strings[ol+'_str'] for ol in _title_list])+' '+utils.resource_strings['month_str']
    option_list = [utils.resource_strings[ol+'_str']+' '+utils.resource_strings['month_str'] for ol in _title_list]
    lm_dlg = OptionDialog(title=_title,option_label='Select options from',options_list=option_list,
                         multi_selection=False,default_options=1)
    lm_dlg.show()
    sys.exit(App.exec())
    #"""